"""

    """

import importlib

import pandas as pd

from src.mirutil import df
from itertools import product


importlib.reload(df_utils)

from src.mirutil.df import *


##
fp0 = '/Users/mahdi/Dropbox/1-git-dirs/PyCharm/u-d0-FirmTicker-MonthlySales/Tables/232760-0.xlsx'
fp1 = '/Users/mahdi/Dropbox/1-git-dirs/PyCharm/u-d0-FirmTicker-MonthlySales/Tables/232760-1.xlsx'
fp2 = '/Users/mahdi/Dropbox/1-git-dirs/PyCharm/u-d0-FirmTicker-MonthlySales/Tables/232768-0.xlsx'

df = pd.read_excel(fp0)
df1 = pd.read_excel(fp1)
df2 = pd.read_excel(fp2)

##
has_extra_data(df , df1)

##
dfs = [df , df1 , df2]

##
l = drop_dup_and_sub_dfs(dfs)
l

##



"""

  """

def ret_clusters_indices(iterable , cluster_size = 100) :
    """ clusters the iterable into clusters of size cluster_size
        and returns the start and end indices of each cluster
        in a list of tuples of start and end indices

    :param iterable: the iterable to be clustered
    :param cluster_size: the size of each cluster
    :return: list of tuples of start and end indices of each cluster
    """
    intdiv = len(iterable) // cluster_size

    cis = [x * cluster_size for x in range(0 , intdiv + 1)]

    if len(cis) > 1 :
        if cis[-1] != len(iterable) :
            cis.append(cis[-1] + len(iterable) % cluster_size)
    else :
        cis = [0 , len(iterable)]
        if cis == [0 , 0] :
            cis = [0]

    se_tuples = []
    for _i in range(len(cis) - 1) :
        se = (cis[_i] , cis[_i + 1])
        se_tuples.append(se)

    prt = se_tuples[0 :3]
    _ = [print(x) for x in prt]

    print('..')

    prt = se_tuples[-2 :]
    _ = [print(x) for x in prt]
    return se_tuples

def print_list_as_dict_fmt(lst) :
    """ prints the list as a dictionary format with the elements as keys"""
    for item in lst :
        print('"' + item + '" : None ,')

def contains_any_of_list(container , ls) :
    """ returns True if any of the elements of ls is in the container. """
    for el in ls :
        if el in container :
            return True
    return False



"""

    """

import asyncio
from dataclasses import dataclass
from functools import partial

import nest_asyncio
from aiohttp import ClientSession
from lxml.etree import XMLSyntaxError
from requests_html import AsyncHTMLSession
from pyppeteer.errors import TimeoutError
from aiohttp.client_exceptions import ClientConnectorError

from .files import write_txt_to_file_async
from .const import Const


nest_asyncio.apply()

cte = Const()

@dataclass
class RGetReqAsync :
    status: int
    headers: dict
    cnt: (bytes , None)

async def _get_req_async(url ,
                         headers = cte.headers ,
                         params = None ,
                         trust_env = False ,
                         verify_ssl = True ,
                         timeout = None) :

    async with ClientSession(trust_env = trust_env) as s :

        try :
            r = await s.get(url ,
                            headers = headers ,
                            params = params ,
                            verify_ssl = verify_ssl ,
                            timeout = timeout)

            return RGetReqAsync(status = r.status ,
                                headers = r.headers ,
                                cnt = await r.read())

        except ClientConnectorError as e :
            print(e)
            return RGetReqAsync(status = r.status ,
                                headers = r.headers ,
                                cnt = None)

async def get_reqs_async(urls ,
                         headers = cte.headers ,
                         params = None ,
                         trust_env = False ,
                         verify_ssl = True ,
                         timeout = None) :

    fu = partial(_get_req_async ,
                 headers = headers ,
                 params = params ,
                 trust_env = trust_env ,
                 verify_ssl = verify_ssl ,
                 timeout = timeout)

    co_tasks = [fu(x) for x in urls]
    return await asyncio.gather(*co_tasks)

async def get_texts_async(urls ,
                          headers = None ,
                          trust_env = False ,
                          params = None ,
                          verify_ssl = True ,
                          timeout = None) :

    fu = partial(get_reqs_async ,
                 headers = headers ,
                 params = params ,
                 trust_env = trust_env ,
                 verify_ssl = verify_ssl ,
                 timeout = timeout)

    return await fu(urls)

async def get_jsons_async(urls ,
                          content_type = None ,
                          headers = cte.headers ,
                          params = None ,
                          trust_env = False ,
                          verify_ssl = True ,
                          timeout = None , ) :

    fu = partial(get_reqs_async ,
                 headers = headers ,
                 params = params ,
                 trust_env = trust_env ,
                 verify_ssl = verify_ssl ,
                 timeout = timeout)

    rs = await fu(urls)

    return [await rs.json(content_type = content_type) for rs in rs]

# getting & saving rendered Htmls async funcs

@dataclass
class RGetAndRender :
    status: int
    headers: dict
    html: (str , None)

async def get_and_render_js_by_requests_html_async(url ,
                                                   headers = cte.headers ,
                                                   params = None ,
                                                   verify = True ,
                                                   timeout = None) :
    ret = RGetAndRender

    ses = AsyncHTMLSession()

    r = await ses.get(url ,
                      headers = headers ,
                      params = params ,
                      verify = verify ,
                      timeout = timeout)

    try :
        await r.html.arender()
        return ret(status = r.status_code ,
                   headers = r.headers ,
                   html = r.html.html)

    except (XMLSyntaxError , TimeoutError) as e :
        print(e)
        return ret(status = r.status_code , headers = r.headers , html = None)

async def get_a_rendered_html_and_save_async(url ,
                                             fp ,
                                             headers = cte.headers ,
                                             params = None ,
                                             verify = True ,
                                             timeout = None) :
    fu = get_and_render_js_by_requests_html_async
    ou = await fu(url ,
                  headers = headers ,
                  params = params ,
                  verify = verify ,
                  timeout = timeout)

    outer_html = ou.html
    if outer_html :
        await write_txt_to_file_async(outer_html , fp)

async def get_rendered_htmls_and_save_async(urls ,
                                            fps ,
                                            headers = cte.headers ,
                                            params = None ,
                                            verify = True ,
                                            timeout = None) :
    fu = partial(get_a_rendered_html_and_save_async ,
                 headers = headers ,
                 params = params ,
                 verify = verify ,
                 timeout = timeout)

    co_tasks = [fu(x , y) for x , y in zip(urls , fps)]
    return await asyncio.gather(*co_tasks)



"""

    """


class Const :
    headers = {
            'User-Agent' : 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.95 Safari/537.36'
            }
